import { useAuthStore } from '@/stores'
import { getApiUrl } from './config-loader';

export const fetchWrapper = {
    get: request('GET'),
    post: request('POST'),
    put: request('PUT'),
    delete: request('DELETE')
}

function request(method: string) {
    return (url: string, body: object | undefined = undefined) => {
        const requestOptions: RequestInit = {
            body: undefined,
            method,
            headers: getAuthHeader(url) || {} as Record<string, string>
        };
        if (body) {
            (requestOptions.headers as Record<string, string>)['Content-Type'] = 'application/json';
            requestOptions.body = JSON.stringify(body);
        }
        return fetch(url, requestOptions).then(handleResponse);
    }
}

async function handleResponse(response: Response) {
    const isJson = response.headers?.get('content-type')?.includes('application/json');
    const data = isJson ? await response.json() : null;

    // check for error response
    if (!response.ok) {
        const { user } = useAuthStore();
        if ([401, 403].includes(response.status) && user) {
            // auto logout if 401 Unauthorized or 403 Forbidden response
            //logout();
        }

        // get error message from body or default to response status
        const error = (data && data.error) || response.status;
        return Promise.reject(error);
    }

    return data;
}

function getAuthHeader(url: string): Record<string, string> {
    const { auth } = useAuthStore();
    const isLoggedIn: boolean = !!auth?.token;
    const apiUrl: string = getApiUrl();
    const isApiUrl = url.startsWith(apiUrl);
    if (isLoggedIn && isApiUrl) {
        return { Authorization: `Bearer ${auth.token}`}
    } else {
        return {};
    }
}